/*HEADER**********************************************************************
*
* Copyright 2013 Freescale Semiconductor, Inc.
*
* This software is owned or controlled by Freescale Semiconductor.
* Use of this software is governed by the Freescale MQX RTOS License
* distributed with this Material.
* See the MQX_RTOS_LICENSE file distributed for more details.
*
* Brief License Summary:
* This software is provided in source form for you to use free of charge,
* but it is not open source software. You are allowed to use this software
* but you cannot redistribute it or derivative works of it in source form.
* The software may be used only in connection with a product containing
* a Freescale microprocessor, microcontroller, or digital signal processor.
* See license agreement file for full license terms including other
* restrictions.
*****************************************************************************
*
* Comments:
*   This file contains the source for the sd2gpio example program.
*
*
*END************************************************************************/

#include "mqx.h"
#include "bsp.h"

/* Function prototypes */
void main_task(uint32_t);

void lcd_init(void);
void lcd_instruction(uint8_t value);
void lcd_data(uint8_t value);
void lcd_send_string(uint8_t *strptr);
void lcd_enable(void);
void lcd_write(uint8_t value);

/* Macro definitions */
#define	  HOME 	  0x80		    		
#define   NXTL	  0xC0
#define   CLS	  0x01

/* Task template */
const TASK_TEMPLATE_STRUCT  MQX_template_list[] =
{
    /* Task Index, Function,  Stack, Priority, Name,   Attributes,          Param, Time Slice */
    { 1,           main_task, 1500,  8,        "main", MQX_AUTO_START_TASK, 0,     0 },
    { 0 }
};

/* Global variables */
/* Structures holding information about specific pin */
LWGPIO_STRUCT sd2gpio0, sd2gpio1, sd2gpio2, sd2gpio3, sd2gpio4, sd2gpio5;

const uint8_t initLCD[8]={0x02, 0x28, 0x0C, 0x06, 0x01, 0x80, 0x00};
const uint8_t message1[] = "  Hello World!";
const uint8_t message2[] = " MQX on i.MX6SX";


/******************************************************************************
*
* Task Name    : main_task
* Comments     : The main task initializes SD3 pines as GPIOs in output mode
*
*
******************************************************************************/
void main_task
    (
        uint32_t initial_data
    )
{
/******************************************************************************
    Open the BSP_SD2GPIO0 to BSP_SD2GPIO5 pins as outputs.
******************************************************************************/
    /* initialize lwgpio handles (sd2gpio0 to sd2gpio5) for BSP_SD2GPIO0 
     * to BSP_SD2GPIO5 pins in default state
     * (defined in mqx/source/bsp/<bsp_name>/<bsp_name>.h file)
     */

    if (!lwgpio_init(&sd2gpio0, BSP_SD2GPIO0, LWGPIO_DIR_OUTPUT, LWGPIO_VALUE_NOCHANGE))
    {
        printf("Initializing SD2GPIO0 GPIO as output failed.\n");
        _task_block();
    }
    /* swich pin functionality (MUX) to GPIO mode */
    lwgpio_set_functionality(&sd2gpio0, BSP_SD2GPIO0_MUX_GPIO);

    if (!lwgpio_init(&sd2gpio1, BSP_SD2GPIO1, LWGPIO_DIR_OUTPUT, LWGPIO_VALUE_NOCHANGE))
    {
        printf("Initializing SD2GPIO1 GPIO as output failed.\n");
        _task_block();
    }
    /* swich pin functionality (MUX) to GPIO mode */
    lwgpio_set_functionality(&sd2gpio1, BSP_SD2GPIO0_MUX_GPIO);

    if (!lwgpio_init(&sd2gpio2, BSP_SD2GPIO2, LWGPIO_DIR_OUTPUT, LWGPIO_VALUE_NOCHANGE))
    {
        printf("Initializing SD2GPIO2 GPIO as output failed.\n");
        _task_block();
    }
    /* swich pin functionality (MUX) to GPIO mode */
    lwgpio_set_functionality(&sd2gpio2, BSP_SD2GPIO2_MUX_GPIO);

    if (!lwgpio_init(&sd2gpio3, BSP_SD2GPIO3, LWGPIO_DIR_OUTPUT, LWGPIO_VALUE_NOCHANGE))
    {
        printf("Initializing SD2GPIO3 GPIO as output failed.\n");
        _task_block();
    }
    /* swich pin functionality (MUX) to GPIO mode */
    lwgpio_set_functionality(&sd2gpio3, BSP_SD2GPIO3_MUX_GPIO);

    if (!lwgpio_init(&sd2gpio4, BSP_SD2GPIO4, LWGPIO_DIR_OUTPUT, LWGPIO_VALUE_NOCHANGE))
    {
        printf("Initializing SD2GPIO4 GPIO as output failed.\n");
        _task_block();
    }
    /* swich pin functionality (MUX) to GPIO mode */
    lwgpio_set_functionality(&sd2gpio4, BSP_SD2GPIO4_MUX_GPIO);

    if (!lwgpio_init(&sd2gpio5, BSP_SD2GPIO5, LWGPIO_DIR_OUTPUT, LWGPIO_VALUE_NOCHANGE))
    {
        printf("Initializing SD2GPIO5 GPIO as output failed.\n");
        _task_block();
    }
    /* swich pin functionality (MUX) to GPIO mode */
    lwgpio_set_functionality(&sd2gpio5, BSP_SD2GPIO5_MUX_GPIO);


    printf("\n===================== SD2GPIO Example =====================\n");
    printf("\n*********** LCD 16x2 connection on 4-bit mode *************\n");

    /* LCD initialization and sending messages */
    lcd_init();
    lcd_send_string(&message1[0]);
    lcd_instruction(NXTL);
    lcd_send_string(&message2[0]);

    printf("\n\n~~~~~~~ DEMO COMPLETED. Hello World! MQX on i.MX6SX ~~~~~~~\n");

    while (TRUE)
    {
      /* Optional execution to do on the main loop */
    }
}


/******************************************************************************
*
* Function Name  : lcd_init
* Comments       : Initializes the LCD on 4-bit mode
*
******************************************************************************/
void lcd_init(void)
{
  uint8_t strindex=0;

  /* Ensuring the Enable is not asserted */
  lwgpio_set_value(&sd2gpio5, LWGPIO_VALUE_LOW);

  while(initLCD[strindex])
  {
    lcd_instruction(initLCD[strindex]);
    strindex++;
  }
}

/******************************************************************************
*
* Function Name  : lcd_instruction
* Comments       : Sends instructions to LCD (RS = 0)
*
******************************************************************************/
void lcd_instruction(uint8_t value)
{
  lwgpio_set_value(&sd2gpio4, LWGPIO_VALUE_LOW); /* SD2GPIO4 used as RS */		
	
  lcd_write( value&0xF0 );
  lcd_enable();
	
  lcd_write( (value<<4)&0xF0 );
  lcd_enable();
}

/******************************************************************************
*
* Function Name  : lcd_data
* Comments       : Sends data to LCD (RS = 1)
*
******************************************************************************/
void lcd_data(uint8_t value)
{
  lwgpio_set_value(&sd2gpio4, LWGPIO_VALUE_HIGH); /* SD2GPIO4 used as RS */		
	
  lcd_write( value&0xF0 );
  lcd_enable();
	
  lcd_write( (value<<4)&0xF0 );
  lcd_enable();
}

/******************************************************************************
*
* Function Name  : lcd_send_string
* Comments       : Sends a string of characters to LCD
*
******************************************************************************/
void lcd_send_string(uint8_t *strptr)
{
  while(*strptr)
  {
    lcd_data(*strptr);
    strptr++;
  }
}

/******************************************************************************
*
* Function Name  : lcd_enable
* Comments       : Sends the enable pulse to LCD
*
******************************************************************************/
void lcd_enable(void)
{
  lwgpio_set_value(&sd2gpio5, LWGPIO_VALUE_HIGH); /* SD2GPIO5 used as ENABLE */
  _time_delay(5);
  lwgpio_set_value(&sd2gpio5, LWGPIO_VALUE_LOW); /* SD2GPIO5 used as ENABLE */
}

/******************************************************************************
*
* Function Name  : lcd_write
* Comments       : Sends the bits to the LCD bus
*
******************************************************************************/
void lcd_write(uint8_t value)
{
  //Bit 7
  if (value&0x80) lwgpio_set_value(&sd2gpio3, LWGPIO_VALUE_HIGH);
  else  lwgpio_set_value(&sd2gpio3, LWGPIO_VALUE_LOW); 
  
  //Bit 6
  if (value&0x40) lwgpio_set_value(&sd2gpio2, LWGPIO_VALUE_HIGH);
  else  lwgpio_set_value(&sd2gpio2, LWGPIO_VALUE_LOW); 
  
  //Bit 5
  if (value&0x20) lwgpio_set_value(&sd2gpio1, LWGPIO_VALUE_HIGH);
  else  lwgpio_set_value(&sd2gpio1, LWGPIO_VALUE_LOW); 

  //Bit 4
  if (value&0x10) lwgpio_set_value(&sd2gpio0, LWGPIO_VALUE_HIGH);
  else  lwgpio_set_value(&sd2gpio0, LWGPIO_VALUE_LOW); 
}

/* EOF */
